\name{pmExpectation}
\alias{pmExpectation}
\alias{pmExpectation-methods}
\alias{pmExpectation,ANY,ANY-method}
\alias{pmExpectation,matrix,missing-method}
\alias{pmExpectation,matrix,vector-method}
\alias{pmExpectation,vector,missing-method}
\alias{pmExpectation,vector,vector-method}
\alias{pmExpectation,missing,vector-method}
\docType{methods}
\title{Methods: Partial Moments Expectation}
\description{
Implements methods for calculating the partial moments either analytically for a given distribution, by sample given the data or resampling
with either the bootstrap or jackknife.\cr
The analytical method, which implements both a quadrature based integration and a trapezium approximation, requires that the user provide 
the name of the  distribution for which a density method exists (\sQuote{d} followed by the distribution name) as well as a fit object 
which should be the second argument to the density method. The sample method is straightforward, requiring only a dataset of differences or returns. 
This function is vectorized so that it can take in a multivariate dataset, distribution list or any combination of the two (see the examples for
more details).
}
\usage{
pmExpectation(data, f, method="sample", fit, threshold, moment=0, type="lower", 
			standardize=FALSE, analytical = "quadrature", n.boot=1000, subdivisions=1000, 
			tailThresh= 10, data.names)
}
\arguments{
  \item{data}{A data matrix or vector only required when using the sample method.}
  \item{f}{ A character vector of distribution names for which density methods exist and takes a fit object as a second argument.}
  \item{method}{ A character vector containing the method of calculating the expectation. Valid methods are \dQuote{analytical}, \dQuote{sample}, 
  \dQuote{boot} for the bootstrap and \dQuote{jack} for the jackknife estimation methods.}
  \item{fit}{ The fit object if any method chosen is \dQuote{analytical}. If there are multiple distributions specified this should be a
  list with the fit objects.}
  \item{threshold}{ A numeric vector of thresholds for calculating the expectation. Optionally, this can be a single number indicating that all
			calculations use the same threshold. }
  \item{moment}{ A positive numeric vector of moments for calculating the expectation. Optionally, this can be a single number indicating that all
			calculations use the same moment. It is a bad idea to mix moments for multivariate datasets if you are then planning to use the
			results in portfolio optimization.}
  \item{type}{ A character vector indicating which tail is being calculated for the expectation. Valid choices are \dQuote{lower} and \dQuote{upper}.
			Optionally, this can be a single character indicating that all calculations use the same tail. }
  \item{standardize}{ A logical vector indicating whether to standardize the result. Standardization raises raises the partial moment to the power of
			\eqn{1/moment}. Optionally, this can be a single logical entry indicating that all calculations are standardized or not. If you are planning
			to pass the object later to the portolio optimizer (via first calculating the co-moment matrix), this should be set to TRUE and
			the same for all the dataset.}
  \item{analytical}{ A character vector indicating the method to use for the analytical method. Valid choices are \dQuote{quadrature} and \dQuote{trapezium}.}
  \item{n.boot}{ No. of bootstrap replications for the \dQuote{boot} method.}
  \item{subdivisions}{ (Optional) A numeric vector indicating the number of subdivisions to use for the analytical method. Additionally, this can be a single number indicating 
			that all analytical calculations use the same number of subdivisions. }
  \item{tailThresh}{ (Optional) A numeric vector indicating the tail (\eqn{Inf}) cutoff to use for integration. Optionally, this can be a single number indicating that all
			calculations use the same number. This is sometimes required for some density functions which have problem dealing with \eqn{Inf} and \eqn{-Inf}. }
  \item{data.names}{ (Optional) A character vector of data names. If only the sample method is used and this is not provided, the function will look to the
			colnames of the data matrix for names. }
}
\value{
An object of class \code{\linkS4class{PME}}
}
\references{
Fishburn, P.C., Decision and Value Theory, 1964, Wiley, New York\cr
Fishburn, P.C., Mean-Risk Analysis with Risk Associated with Below-Target Returns, 1977, American Economic Review, 67, 116-126\cr
}
\author{Alexios Ghalanos}
\note{
Has method for show. The object can then be passed to the \code{\link{pcmExpectation}} to generate the partial co-moment
matrix used in the quadratic portfolio optimization routine \code{\link{pmSolver}}. For single datasets see the 
\code{\link{pmoments}} method.
}
\examples{
\dontrun{

# First we prepare the analytical solution:
library(RBestFit)
data(djiaret)
# Fit and Test for bestfiting distributions
dow30fit<-apply(as.matrix(djiaret), 2, FUN=function(x) bestfit(x, c("spd",
"ghypuv","Normal2mix"),test="AD", type="univariate"))
# extract the bestfitted distribution object
bdist<-lapply(dow30fit,FUN=function(x) getbestfit(x))
# extract the bestfitted distribution names
bdistnames<-sapply(dow30fit,FUN=function(x) (x@bestfitted),simplify=TRUE)
##############################################
# Partial Moments Expectations (pmExpectation)
##############################################
#--------------------------------------------------------------------
# Test1: 1 Dataset, 1 Method ( "sample" ), many moments
lpm1<-pmExpectation(data=as.matrix(djiaret[,1]), method="sample", 
		threshold=-0.01, moment=c(0:10), type="lower", 
		standardize=TRUE, data.names = colnames(djiaret)[1])

show(lpm1)
plot(lpm1)
#----------------------------------------------------------------------------------
# Test2: 1 Dataset, 1 Method ( "analytical/quadraure" ), many moments, 
# one distribution (bestfitted)
# REM: fit must be a list object
lpm2<-pmExpectation(f=bdistnames[2], method="analytical", fit=list(bdist[[2]]), 
		threshold=-0.01, moment=c(0:10), type="lower", standardize=TRUE, 
		data.names = colnames(djiaret)[2])

show(lpm2)
plot(lpm2)
#----------------------------------------------------------------------------------
# Test3: 1 Dataset, 1 Method ( "analytical/quadraure" ), 1 moments, many distributions
# rem fit must be a list object
# Note: You cannot have multiple moments for multiple analytic distributions
# in that case you should run seperate routines.
fx=dow30fit[[2]]@distributions
fitx = lapply(dow30fit[[2]]@gof@test,FUN=function(x) x$fit)
# check to see that fit object is list:
is(fitx) # ok

lpm3<-pmExpectation(f=fx, method="analytical", fit=fitx, threshold=-0.01, 
		moment=1, type="lower", standardize=TRUE, 
		data.names = colnames(djiaret)[2])

show(lpm3)
plot(lpm3)
#----------------------------------------------------------------------------------
# Test4: 1 Dataset, 1 Method ( "analytical/trapezium" ), many moments
# rem fit must be a list object
lpm4<-pmExpectation(f=bdistnames[2], method="analytical", analytical="trapezium", 
		fit=list(bdist[[2]]), threshold=-0.01, moment=c(0:10), 
		type="lower", standardize=TRUE, data.names = colnames(djiaret)[2])

show(lpm4)
plot(lpm4)
#----------------------------------------------------------------------------------
# Test5: 1 Dataset, 1 Method ( "analytical/trapezium" ), 1 moments, many distributions
# rem fit must be a list object
fx=dow30fit[[2]]@distributions
fitx = lapply(dow30fit[[2]]@gof@test,FUN=function(x) x$fit)

lpm5<-pmExpectation(f=fx, method="analytical", analytical="trapezium", 
		fit=fitx, threshold=-0.01, moment=1, type="lower", 
		standardize=TRUE, data.names = colnames(djiaret)[2])

show(lpm5)
plot(lpm5)
#----------------------------------------------------------------------------------
# Test6: 1 Dataset, 1 Method ( "boot" ), many moments
# rem fit must be a list object
lpm6<-pmExpectation(data=as.matrix(djiaret[,1]), method="boot", threshold=-0.01, 
		moment=c(0:10), type="lower", standardize=TRUE, 
		data.names = colnames(djiaret)[1])

show(lpm6)
plot(lpm6)
#----------------------------------------------------------------------------------
# Test7: 1 Dataset, 1 Method ( "jack" ), many moments
# rem fit must be a list object
lpm7<-pmExpectation(data=as.matrix(djiaret[,1]), method="jack", threshold=-0.01, 
		moment=c(0:10), type="lower", standardize=TRUE, 
		data.names = colnames(djiaret)[1])

show(lpm7)
plot(lpm7)
#----------------------------------------------------------------------------------
# Test8: Many Dataset, 1 Method ( "boot" ), one moment
# rem fit must be a list object
lpm8<-pmExpectation(data=as.matrix(djiaret), method="boot", threshold=-0.01, 
		moment=0, type="lower", standardize=TRUE, 
		data.names = colnames(djiaret))

show(lpm8)
plot(lpm8)
#----------------------------------------------------------------------------------
# Test9: Many Dataset, Many Methods, one moment
lpm9<-pmExpectation(f=bdistnames, data=as.matrix(djiaret), 
		method=c(rep("sample",10),rep("analytical",10),rep("boot",10)), 
		fit=bdist, threshold=-0.01, moment=2, type="lower", 
		standardize=TRUE, data.names = colnames(djiaret))

show(lpm9)
plot(lpm9)
#----------------------------------------------------------------------------------
# Test10: One Dataset, Many Methods, One Moment
fx=dow30fit[[1]]@distributions
fitx = lapply(dow30fit[[1]]@gof@test,FUN=function(x) x$fit)

lpm10<-pmExpectation(f=fx, data=as.matrix(djiaret[,1]), method=c("sample",
		"analytical","boot"), fit=fitx, threshold=-0.01, moment=2, 
		type="lower", standardize=TRUE, analytical = "quadrature",
		data.names = colnames(djiaret)[1]))
show(lpm10)
plot(lpm10)
#----------------------------------------------------------------------------------
# Test11: One Dataset, Many Methods, Many Moments
fx=dow30fit[[1]]@distributions
fitx = lapply(dow30fit[[1]]@gof@test,FUN=function(x) x$fit)

lpm11<-pmExpectation(f=fx, data=as.matrix(djiaret[,1]), method=c("sample",
		"analytical","boot"), fit=fitx, threshold=-0.01, moment=c(0,1,
		2,3,3.5,4), type="lower", standardize=TRUE,
		data.names = colnames(djiaret)[1])
show(lpm11)
plot(lpm11)
#----------------------------------------------------------------------------------
# Extractor Functions are effectively as.data.frame(PME)
# examples:
as.data.frame(lpm1)
as.data.frame(lpm2)
as.data.frame(lpm3)
as.data.frame(lpm4)
as.data.frame(lpm5)
as.data.frame(lpm11)
}}
\keyword{methods}